@@ -6,7 +6,7 @@
 //! use four_bar::{FourBar, Mechanism};
 //!
 //! // A four-bar mechanism example
-//! let m = Mechanism::four_bar(FourBar::default());
+//! let m = Mechanism::four_bar(&FourBar::default());
 //! // Get the trajectory of the coupler point
 //! let path = m.four_bar_loop(0., 360);
 //! ```
@@ -20,5 +20,4 @@ mod mechanism;
 pub mod plot;
 mod point;
 pub mod synthesis;
-#[cfg(test)]
-mod tests;
+pub mod tests;
