@@ -1,6 +1,5 @@
 //! The functions used to plot the curve and synthesis result.
 
-use metaheuristics_nature::Report;
 use plotters::prelude::*;
 use std::path::Path;
 
@@ -11,24 +10,21 @@ const FONT: &str = if cfg!(windows) {
 };
 
 /// Plot the synthesis history.
-pub fn plot_history<P>(history: &[Report], path: P)
+pub fn plot_history<P>(history: &[f64], path: P)
 where
     P: AsRef<Path>,
 {
     let root = SVGBackend::new(&path, (1600, 900)).into_drawing_area();
     root.fill(&WHITE).unwrap();
-    let gen_max = history.iter().map(|r| r.gen).max().unwrap();
-    let (max_best, _) = find_extreme(history.iter().map(|r| r.best_f));
-    let (max_average, _) = find_extreme(history.iter().map(|r| r.average));
+    let (max_best, _) = find_extreme(history.iter().cloned());
     let mut chart = ChartBuilder::on(&root)
         .caption("History", (FONT, 50))
         .x_label_area_size(45)
         .y_label_area_size(50)
         .right_y_label_area_size(50)
         .margin(20)
-        .build_cartesian_2d(0..gen_max, 0f64..max_best)
-        .unwrap()
-        .set_secondary_coord(0..gen_max, 0f64..max_average);
+        .build_cartesian_2d(0..history.len(), 0f64..max_best)
+        .unwrap();
     chart
         .configure_mesh()
         .disable_x_mesh()
@@ -39,31 +35,15 @@ where
         .y_desc("Best Fitness")
         .draw()
         .unwrap();
-    chart
-        .configure_secondary_axes()
-        .label_style((FONT, 20))
-        .axis_desc_style((FONT, 20))
-        .y_desc("Average Fitness")
-        .draw()
-        .unwrap();
     const BLUE: RGBColor = RGBColor(118, 182, 222);
     chart
         .draw_series(LineSeries::new(
-            history.iter().map(|r| (r.gen, r.best_f)),
+            history.iter().cloned().enumerate(),
             BLUE.stroke_width(5),
         ))
         .unwrap()
         .label("Best Fitness")
         .legend(|(x, y)| PathElement::new([(x, y), (x + 20, y)], BLUE));
-    const GREEN: RGBColor = RGBColor(187, 222, 13);
-    chart
-        .draw_secondary_series(LineSeries::new(
-            history.iter().map(|r| (r.gen, r.average)),
-            GREEN.stroke_width(5),
-        ))
-        .unwrap()
-        .label("Average Fitness")
-        .legend(|(x, y)| PathElement::new([(x, y), (x + 20, y)], GREEN));
     chart
         .configure_series_labels()
         .label_font((FONT, 30))
@@ -78,8 +58,10 @@ pub fn plot_curve<P>(title: &str, curves: &[(&str, &[[f64; 2]])], path: P)
 where
     P: AsRef<Path>,
 {
-    let iter = curves.iter().flat_map(|(_, c)| c.iter().flat_map(|c| *c));
-    let (p_max, p_min) = find_extreme(iter);
+    let xs = curves.iter().flat_map(|(_, c)| c.iter().map(|c| c[0]));
+    let ys = curves.iter().flat_map(|(_, c)| c.iter().map(|c| c[1]));
+    let (x_max, x_min) = find_extreme(xs);
+    let (y_max, y_min) = find_extreme(ys);
     let root = SVGBackend::new(&path, (1000, 1000)).into_drawing_area();
     root.fill(&WHITE).unwrap();
     let mut chart = ChartBuilder::on(&root)
@@ -87,7 +69,7 @@ where
         .x_label_area_size(40)
         .y_label_area_size(40)
         .margin(20)
-        .build_cartesian_2d(p_min..p_max, p_min..p_max)
+        .build_cartesian_2d((x_min - 4.)..x_max + 4., (y_min - 4.)..y_max + 4.)
         .unwrap();
     chart
         .configure_mesh()
